@@ -75,7 +75,7 @@ def split_host(hoststring):
         host = arr[0]
         if len(arr) == 2:
             port = arr[1]
-        
+
     return (host, port, user, passwd)
 
 def get_proxy_info(proxy):
@@ -86,9 +86,9 @@ def get_proxy_info(proxy):
     if len(arr) == 2:
         # scheme found, strip it
         proxy = arr[1]
-    
+
     return split_host(proxy)
-        
+
 
 class MalformedURIError(IOError):
     pass
@@ -126,9 +126,9 @@ class Server:
     _transport_class_https = transports.SafeTransport
     _transport_class_proxy = transports.ProxyTransport
     _transport_class_https_proxy = transports.SafeProxyTransport
-    def __init__(self, uri, transport=None, encoding=None, verbose=0, 
+    def __init__(self, uri, transport=None, encoding=None, verbose=0,
         proxy=None, username=None, password=None, refreshCallback=None,
-        progressCallback=None):
+        progressCallback=None, timeout=None):
         # establish a "logical" server connection
 
         #
@@ -149,7 +149,7 @@ class Server:
 
                 if pw is not None and password is None:
                     password = pw
-                    
+
         self._uri = uri
         self._refreshCallback = None
         self._progressCallback = None
@@ -157,6 +157,7 @@ class Server:
         self._proxy = proxy
         self._username = username
         self._password = password
+        self._timeout = timeout
 
         if len(__version__.split()) > 1:
             self.rpc_version = __version__.split()[1]
@@ -167,14 +168,15 @@ class Server:
 
         if transport is None:
             self._allow_redirect = 1
-            transport = self.default_transport(self._type, proxy, username, password)
+            transport = self.default_transport(self._type, proxy, username,
+                    password, timeout)
         else:
             #
             # dont allow redirect on unknow transports, that should be
             # set up independantly
             #
             self._allow_redirect = 0
-            
+
         self._redirected = None
         self.use_handler_path = 1
         self._transport = transport
@@ -193,19 +195,20 @@ class Server:
 
         self._headers = UserDictCase()
 
-    def default_transport(self, type, proxy=None, username=None, password=None):
+    def default_transport(self, type, proxy=None, username=None, password=None,
+            timeout=None):
         if proxy:
             if type == 'https':
-                transport = self._transport_class_https_proxy(proxy, 
-                    proxyUsername=username, proxyPassword=password)
+                transport = self._transport_class_https_proxy(proxy,
+                    proxyUsername=username, proxyPassword=password, timeout=timeout)
             else:
-                transport = self._transport_class_proxy(proxy, 
-                    proxyUsername=username, proxyPassword=password)
+                transport = self._transport_class_proxy(proxy,
+                    proxyUsername=username, proxyPassword=password, timeout=timeout)
         else:
             if type == 'https':
-                transport = self._transport_class_https()
+                transport = self._transport_class_https(timeout=timeout)
             else:
-                transport = self._transport_class()
+                transport = self._transport_class(timeout=timeout)
         return transport
 
     def allow_redirect(self, allow):
@@ -339,7 +342,7 @@ class Server:
             return tuple(result)
 
     def _request(self, methodname, params):
-        """ Call a method on the remote server 
+        """ Call a method on the remote server
             we can handle redirections. """
         # the loop is used to handle redirections
         redirect_response = 0
@@ -358,11 +361,11 @@ class Server:
                 self._transport.set_header(k, v)
 
             self._transport.add_header("X-Info",
-                'RPC Processor (C) Red Hat, Inc (version %s)' % 
+                'RPC Processor (C) Red Hat, Inc (version %s)' %
                 self.rpc_version)
             # identify the capability set of this client to the server
             self._transport.set_header("X-Client-Version", 1)
-            
+
             if self._allow_redirect:
                 # Advertise that we follow redirects
                 #changing the version from 1 to 2 to support backward compatibility
@@ -402,12 +405,12 @@ class Server:
 
             if not self._allow_redirect:
                 raise InvalidRedirectionError("Redirects not allowed")
-                                
+
             if self._verbose:
                 print "%s redirected to %s" % (self._uri, self._redirected)
 
             typ, uri = urllib.splittype(self._redirected)
-            
+
             if typ != None:
                 typ = typ.lower()
             if typ not in ("http", "https"):
@@ -431,7 +434,7 @@ class Server:
             # set up the parameters on the new transport
             del self._transport
             self._transport = self.default_transport(typ, self._proxy,
-                                     self._username, self._password)
+                    self._username, self._password, self._timeout)
             self.set_progress_callback(self._progressCallback)
             self.set_refresh_callback(self._refreshCallback)
             self.set_buffer_size(self._bufferSize)
@@ -446,7 +449,7 @@ class Server:
         if isinstance(response, transports.File):
             # Just return the file
             return response
-            
+
         # an XML-RPC encoded data structure
         if isinstance(response, TupleType) and len(response) == 1:
             response = response[0]
@@ -510,7 +513,7 @@ class Server:
         self._lang = lang
         if self._transport and hasattr(self._transport, "setlang"):
             self._transport.setlang(lang)
-        
+
     # Sets the CA chain to be used
     def use_CA_chain(self, ca_chain = None):
         raise NotImplementedError, "This method is deprecated"
@@ -519,7 +522,7 @@ class Server:
         self._trusted_cert_files.append(certfile)
         if self._transport and hasattr(self._transport, "add_trusted_cert"):
             self._transport.add_trusted_cert(certfile)
-        
+
     def close(self):
         if self._transport:
             self._transport.close()
@@ -529,14 +532,15 @@ class Server:
 class GETServer(Server):
     def __init__(self, uri, transport=None, proxy=None, username=None,
             password=None, client_version=2, headers={}, refreshCallback=None,
-            progressCallback=None):
-        Server.__init__(self, uri, 
+            progressCallback=None, timeout=None):
+        Server.__init__(self, uri,
             proxy=proxy,
             username=username,
             password=password,
             transport=transport,
             refreshCallback=refreshCallback,
-            progressCallback=progressCallback)
+            progressCallback=progressCallback,
+            timeout=timeout)
         self._client_version = client_version
         self._headers = headers
         # Back up the original handler, since we mangle it
@@ -555,12 +559,12 @@ class GETServer(Server):
 
         #save the constructed handler in case of redirect
         self.send_handler = self._handler
-        
+
         # Add headers
         #override the handler to replace /XMLRPC with pkg path
         if self._redirected and not self.use_handler_path:
            self._handler = self._new_req_body()
-            
+
         for h, v in self._headers.items():
             self._transport.set_header(h, v)
 
@@ -584,7 +588,7 @@ class GETServer(Server):
         type, tmpuri = urllib.splittype(self._redirected)
         site, handler = urllib.splithost(tmpuri)
         return handler
-    
+
     def set_range(self, offset=None, amount=None):
         if offset is not None:
             try:
@@ -602,7 +606,7 @@ class GETServer(Server):
 
             if amount <= 0:
                 raise RangeError("Invalid value `%s' for amount" % amount)
-                
+
         self._amount = amount
         self._offset = offset
 
@@ -613,8 +617,9 @@ class GETServer(Server):
         # magic method dispatcher
         return SlicingMethod(self._request, name)
 
-    def default_transport(self, type, proxy=None, username=None, password=None):
-	ret = Server.default_transport(self, type, proxy=proxy, username=username, password=password)
+    def default_transport(self, type, proxy=None, username=None, password=None,
+            timeout=None):
+	ret = Server.default_transport(self, type, proxy=proxy, username=username, password=password, timeout=timeout)
 	ret.set_method("GET")
 	return ret
 
@@ -631,7 +636,7 @@ def getHeaderValues(headers, name):
             return [headers[name]]
         return []
 
-    return map(lambda x: x.split(':', 1)[1].strip(), 
+    return map(lambda x: x.split(':', 1)[1].strip(),
             headers.getallmatchingheaders(name))
 
 class _Method:
@@ -666,7 +671,7 @@ class SlicingMethod(_Method):
         self._offset = kwargs.get('offset')
         self._amount = kwargs.get('amount')
 
-        # im_self is a pointer to self, so we can modify the class underneath 
+        # im_self is a pointer to self, so we can modify the class underneath
         try:
             self._send.im_self.set_range(offset=self._offset,
                 amount=self._amount)
@@ -682,7 +687,7 @@ class SlicingMethod(_Method):
             pass
 
         return result
-        
+
 
 def reportError(headers):
     """ Reports the error from the headers. """
