@@ -2,7 +2,6 @@
 #
 # munge the headers to produce an error message
 #
-# $Id$
 #
 # USAGE:  $0 SERVER SYSTEMID
 
@@ -42,5 +41,5 @@ except ProtocolError, e:
     print reportError(e.headers)
     print("OK (error above expected)");
     sys.exit(0);
-print("ERROR: Exception didn't occured!");
+print("ERROR: Exception didn't occurred!");
 sys.exit(-1);
